/***********************************************************************
 Name             : Plot.java
 Autor            : Marcel Lattmann
                    Eidgenoessisch Technische Hochschule Zuerich (ETHZ)
 Letzte Aenderung : 23.7.1999

 Einfache Routine zum Plotten von Kurven.
 
 Benoetigt folgende Klassen:
   Matrix.class	          Datentyp Matrix und Lineare Algebra Operationen
   plotText               Datentyp, um Text im Graphen auszugeben

 History:
************************************************************************/

import java.awt.*;
import java.lang.*;
import java.util.Vector;

class Plot extends Canvas {
  /* Plotte die Punkte und das Ausgleichspolynom
     P = Matrix fuer die Datenpunkte.
       P.element[i][0] = x-Koordinate
       P.element[i][1] = y-Koordinate
     M = Matrix mit den Koeffizienten des Ausgleichspolynoms
  */

  // Konstanten
  public static final int RADIUS = 3;   // Radius der Punkte  

  /* Eckpunkte des Koordinatensystems. Muessen public sein, damit die
     Maushandler die Koordinaten der Maus umrechnen koennen.*/

  public int xmax, xmin, ymax, ymin;
  public int xcenter, ycenter;
  public int linesegs;          // Anzahl Liniensegmente fuer das Polygon

  public boolean autoScale=true;
  // Wenn autoScale false ist, dann kann man die Achsen
  // manuell setzen.

  public Matrix P = new Matrix();	
  public Matrix M = new Matrix();	
  public boolean squareAspectRatio = false;
  // wenn squareAspectRatio=true, dann wird eine Square Plot gezeichnet

  // Ein Vektor mit auszugebenden textPlot-Objekten
  public Vector text=new Vector(); 

  Plot() {
    super();
    setBackground(Color.white);
  }

  public void clear(){
    // loesche die Grafik
    P=new Matrix();  // Loesche alle Punkte
    // Loesche Vektor mit den Koeffizienten des Ausgleichspolynoms
    M=new Matrix();
    text=new Vector();
    repaint();

  }

  public void paint(Graphics g) {
    /* draw border, ticks, first 
       checking coordinate system bounds */

    int i,j;
    double x,y,z;

    // Feste Koordinaten fuer die Eckpunkte
    xmin=-10; xmax=+10;
    ymin=-10; ymax=+10;
 
    xcenter = 0;
    ycenter = 0;

    // Ermittle die Groesse des Plots und zeichne das 
    // Koordinatensystem
    Dimension d = getSize();
    int dh = 0;
    if((xmin*xmax)<0){
      xcenter=(int)(Math.abs(((double)xmin)/((double)(xmax-xmin)))*(double)d.width);
    } else
      xcenter = 0;
    if((ymin*ymax)<0){
      ycenter=(int)(Math.abs(((double)ymax)/((double)(ymax-ymin)))*(double)d.height);
    } else
      ycenter = d.height;

    // Rechteck um die Grafik herum
    g.draw3DRect(0,0, d.width-1, d.height-1, true);

    // plotte das Achsenkreuz
    g.setColor(Color.gray);
    g.drawLine(0,d.height/2,d.width,d.height/2);
    g.drawLine(d.width/2,0,d.width/2,d.height);

    // Horizontale Achsen
    for(i=xmin; i<=xmax; i++){
      j=d.width/2+(int)(i*d.width/(xmax-xmin));
      g.drawLine(j,0,j,3);
      g.drawLine(j,d.height/2+1,j,d.height/2-2);
      g.drawLine(j,d.height,j,d.height-3);
      if (i != 0)
        g.drawString(i+"",j-5,d.height/2+13);
    }
    // Vertikale Achsen
    for(i=ymin; i<=ymax; i++){
      j = d.height/2+(int)(i*d.height/(ymax-ymin));
      g.drawLine(0,j,3,j);
      g.drawLine(d.width/2-1,j,d.width/2-4,j);
      g.drawLine(d.width,j,d.width-3,j);
      if (i != 0)
        g.drawString(-i+ "",d.width/2+10,j-3);
    }

    // Plotte die Punkte
    if(P.spalten>1){
      for(i=0; i<P.zeilen; i++){

	x = P.element[i][0]*d.width/(xmax-xmin);
	y = P.element[i][1]*d.height/(ymax-ymin);

	g.setColor(Color.red);    // Punkte rot
	g.fillOval(xcenter+(int)x-RADIUS,ycenter-(int)y-RADIUS,2*RADIUS,2*RADIUS);
      }
    }

    // Plotte die Kurve, sofern sie existiert
    double x2;
    if(M.zeilen>1) {
      double xstep = (double)(xmax-xmin)/linesegs;
      double xx;
      // Zeichne die Kurve schwarz
      g.setColor(Color.black);
      for(xx=xmin; xx<xmax; xx = xx + xstep){
	y = 0;
	z = 0;
	for(j=0; j<M.zeilen; j++){
	  y = y + M.element[j][0]*Math.pow(xx,j);
	  z = z + M.element[j][0]*Math.pow((xx+xstep),j);
	}
	x = xx*d.width/(xmax-xmin) + xcenter;
	x2 = (xx+xstep)*d.width/(xmax-xmin) + xcenter;
        y = ycenter - y*d.height/(ymax-ymin);
        z = ycenter - z*d.height/(ymax-ymin);
	g.drawLine((int)x,(int)y,(int)x2,(int)z);
      }	

      // Plotte die Residuen blau
      g.setColor(Color.blue);
      if(P.spalten>1){
        for(i=0; i<P.zeilen; i++){
 
	  x = P.element[i][0];
  	  y = P.element[i][1];

	  z = 0;
	  for(j=0; j<M.zeilen; j++){
	    z = z + M.element[j][0]*Math.pow((double)x,j);
	  }
	  x = x*d.width/(xmax-xmin) + xcenter;
          y = ycenter - y*d.height/(ymax-ymin);
          z = ycenter - z*d.height/(ymax-ymin);
	  g.drawLine((int)x,(int)y,(int)x,(int)z);
        }
      }
    }

    // Gib zum Schluss die Textobjekte aus (sofern welche existieren)
    if(!text.isEmpty()){
      plotText t;
      int k; int a;
      FontMetrics fm;
      for(k=0;k<text.size();k++){
	t=(plotText)text.elementAt(k);
	g.setColor(t.color);
	g.setFont(t.font);
	fm = g.getFontMetrics(t.font);	
	a=fm.getAscent();
	x = t.x*d.width/(xmax-xmin);
	y = t.y*d.height/(ymax-ymin)+a/2;
	g.drawString(t.text,(int)(xcenter+x),(int)(ycenter-y));
      }
    }
  }
}


